<!doctype html>
<html>
    <head>
        <title>Our Funky HTML Page</title>
        <meta name="description" content="Our first page">
        <meta name="keywords" content="html tutorial template">
    </head>
    <body>
        <h1>Hello World!!</h1>
        <p>{{{ counter }}}</p>
        
        <!--  add a button that will post to /counter  -->
        <form action="/counter" method="POST">
            <input type="submit" value="increment">
        </form>
    </body>
</html>
